package edu.indiana.extreme.xbaya.gfac;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.xmlpull.v1.builder.XmlElement;
import org.xmlpull.v1.builder.XmlInfosetBuilder;

import xsul.XmlConstants;
import xsul.wsif.WSIFMessage;
import xsul.wsif.WSIFOperation;
import xsul.wsif.WSIFPort;
import xsul.wsif.impl.WSIFMessageElement;
import xsul.xwsif_runtime.WSIFClient;
import xsul.xwsif_runtime.WSIFRuntime;
import xsul5.MLogger;
import edu.indiana.extreme.xbaya.component.registry.ComponentRegistryException;

/**
 * This is a Simple Web Service client for easy SOAP Messages creation
 * 
 * @author Srinath Perera(hperera@cs.indiana.edu)
 * @author Gopi Kandaswamy (gkandasw@cs.indiana.edu)
 * @author Satoshi Shirasuna
 */
public class SimpleWSClient {

    private static final MLogger logger = MLogger.getLogger();

    private static final XmlInfosetBuilder builder = XmlConstants.BUILDER;

    private String requestNS = GFacRegistryClient.GFAC_NAMESPACE;

    /**
     * @param url
     * @param args
     * @param opName
     * @return The output
     * @throws ComponentRegistryException
     */
    public WSIFMessage sendSOAPMessage(String url, Object[][] args,
            String opName) throws ComponentRegistryException {
        WSIFClient wclient = WSIFRuntime.newClient(url);
        return sendSOAPMessage(wclient, args, opName);
    }

    /**
     * @param wclient
     * @param args
     * @param opName
     * @return The output
     * @throws ComponentRegistryException
     */
    public WSIFMessage sendSOAPMessage(WSIFClient wclient, Object[][] args,
            String opName) throws ComponentRegistryException {

        WSIFPort port = wclient.getPort();

        WSIFOperation operation = port.createOperation(opName);
        WSIFMessage outputMessage = operation.createOutputMessage();
        WSIFMessage faultMessage = operation.createFaultMessage();
        String messageName = operation.createInputMessage().getName();
        XmlElement inputMsgElem = builder.newFragment(this.requestNS,
                messageName);

        for (int i = 0; i < args.length; i++) {
            createMessage((String) args[i][0], args[i][1], inputMsgElem);
        }

        WSIFMessageElement inputMessage = new WSIFMessageElement(inputMsgElem);

        boolean success = operation.executeRequestResponseOperation(
                inputMessage, outputMessage, faultMessage);
        if (success) {
            logger.finest("" + outputMessage);
            return outputMessage;
        } else {
            throw new ComponentRegistryException("Excpetion at server "
                    + faultMessage);
        }
    }

    private void createMessage(String paramName, Object value,
            XmlElement inputMsgElem) throws ComponentRegistryException {
        XmlElement paramsElem = builder.newFragment(this.requestNS, paramName);
        if (value instanceof String) {
            paramsElem.addChild(value);
        } else if (value instanceof Collection) {
            Collection list = (Collection) value;
            Iterator arrayValues = list.iterator();
            while (arrayValues.hasNext()) {
                XmlElement item = builder.newFragment("value");
                item.addChild(arrayValues.next());
                paramsElem.addChild(item);
            }
        } else if (value instanceof ArrayList) {
            Collection list = (Collection) value;
            Iterator arrayValues = list.iterator();
            while (arrayValues.hasNext()) {
                XmlElement item = builder.newFragment("value");
                item.addChild(arrayValues.next());
                paramsElem.addChild(item);
            }
        } else if (value instanceof String[]) {
            String[] list = (String[]) value;
            for (int i = 0; i < list.length; i++) {
                XmlElement item = builder.newFragment("value");
                item.addChild(list[i]);
                paramsElem.addChild(item);
            }
        } else {
            throw new ComponentRegistryException(
                    "Simple WS Client can not handle the value of type "
                            + value);
        }

        inputMsgElem.addElement(paramsElem);
    }

}
